/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                SFS_Port_I2C_GD32VF103C.c                 *
*                GD32VF103C I2C xRAM PORT                  *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"

#include "gd32vf103_rcu.h"
#include "gd32vf103_gpio.h"
#include "gd32vf103_i2c.h"

/***********************************************************
 *   GigaDevice GD32VF103C configuration
 ***********************************************************
 *
 *   PB11 --- I2C1 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PB10 --- I2C1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are required (2.2k) !!!
 **********************************************************/

/*
************************************************************
*                LOWLEVEL READ BYTES
************************************************************
*/
U08 _sfsPort_readBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    while (i2c_flag_get(I2C1, I2C_FLAG_I2CBSY));
    if (2 == len) i2c_ackpos_config(I2C1, I2C_ACKPOS_NEXT);
    i2c_start_on_bus(I2C1);
    while (!i2c_flag_get(I2C1, I2C_FLAG_SBSEND));
    i2c_master_addressing(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C1, I2C_FLAG_ADDSEND));
    i2c_flag_clear(I2C1, I2C_FLAG_ADDSEND);
    while (SET != i2c_flag_get(I2C1, I2C_FLAG_TBE));
    i2c_enable(I2C1);
    for (i = 2; i > 0; i--) {
    	i2c_data_transmit(I2C1,(addr >> (8*(i-1))) & 0xFF);
        while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
    }
    i2c_start_on_bus(I2C1);
    while (!i2c_flag_get(I2C1, I2C_FLAG_SBSEND));
    i2c_master_addressing(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_RECEIVER);
    if (len < 3) i2c_ack_config(I2C1, I2C_ACK_DISABLE);
    while (!i2c_flag_get(I2C1, I2C_FLAG_ADDSEND));
    i2c_flag_clear(I2C1, I2C_FLAG_ADDSEND);
    if (1 == len) i2c_stop_on_bus(I2C1);
    i = 0;
    while (len) {
        if (3 == len) {
            while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
            i2c_ack_config(I2C1, I2C_ACK_DISABLE);
        }
        if (2 == len) {
            while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
            i2c_stop_on_bus(I2C1);
        }
        if (i2c_flag_get(I2C1, I2C_FLAG_RBNE)) {
            data[i] = i2c_data_receive(I2C1);
            --len;
            ++i;
        }
    }
    while (I2C_CTL0(I2C1) & 0x0200);
    i2c_ack_config(I2C1, I2C_ACK_ENABLE);
    i2c_ackpos_config(I2C1, I2C_ACKPOS_CURRENT);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WRITE BYTES
************************************************************
*/
U08 _sfsPort_writeBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    while (i2c_flag_get(I2C1, I2C_FLAG_I2CBSY));
    i2c_start_on_bus(I2C1);
    while (!i2c_flag_get(I2C1, I2C_FLAG_SBSEND));
    i2c_master_addressing(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C1, I2C_FLAG_ADDSEND));
    i2c_flag_clear(I2C1, I2C_FLAG_ADDSEND);
    while (SET != i2c_flag_get(I2C1, I2C_FLAG_TBE));
    for (i = 2; i > 0; i--) {
    	i2c_data_transmit(I2C1,(addr >> (8*(i-1))) & 0xFF);
        while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
    }
    for (i = 0; i < len; i++) {
    	i2c_data_transmit(I2C1, data[i]);
    	while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
    }
    i2c_stop_on_bus(I2C1);
    while (I2C_CTL0(I2C1) & 0x0200);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL FILL BYTES
************************************************************
*/
U08 _sfsPort_fillBytes(U32 addr, U08 value, SFS_LONG len)
{
    SFS_LONG i;

    while (i2c_flag_get(I2C1, I2C_FLAG_I2CBSY));
    i2c_start_on_bus(I2C1);
    while (!i2c_flag_get(I2C1, I2C_FLAG_SBSEND));
    i2c_master_addressing(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_TRANSMITTER);
    while (!i2c_flag_get(I2C1, I2C_FLAG_ADDSEND));
    i2c_flag_clear(I2C1, I2C_FLAG_ADDSEND);
    while (SET != i2c_flag_get(I2C1, I2C_FLAG_TBE));
    for (i = 2; i > 0; i--) {
        i2c_data_transmit(I2C1,(addr >> (8*(i-1))) & 0xFF);
        while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
    }
    for (i = 0; i < len; i++) {
    	i2c_data_transmit(I2C1, value);
     	while (!i2c_flag_get(I2C1, I2C_FLAG_BTC));
    }
    i2c_stop_on_bus(I2C1);
    while (I2C_CTL0(I2C1) & 0x0200);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WAIT READY
************************************************************
*/
void _sfsPort_writeReady(void)
{
    U32 s, addr = 0;

    while (1) {
        while (i2c_flag_get(I2C1, I2C_FLAG_I2CBSY));
        i2c_start_on_bus(I2C1);
        while (!i2c_flag_get(I2C1, I2C_FLAG_SBSEND));
        i2c_master_addressing(I2C1, I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK, I2C_TRANSMITTER);
        do {
            s = I2C_STAT0(I2C1);
        } while (0 == (s & (I2C_STAT0_ADDSEND | I2C_STAT0_AERR)));
        if (s & I2C_STAT0_ADDSEND) {
            i2c_flag_clear(I2C1, I2C_FLAG_ADDSEND);
            i2c_stop_on_bus(I2C1);
            return;
        } else {
            i2c_flag_clear(I2C1, I2C_FLAG_AERR);
        }
        i2c_stop_on_bus(I2C1);
        while (I2C_CTL0(I2C1) & 0x0200);
    }
}

/*
************************************************************
*                 INIT I2C-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    rcu_periph_clock_enable(RCU_GPIOB);
    rcu_periph_clock_enable(RCU_I2C1);

    gpio_init(GPIOB, GPIO_MODE_AF_OD, GPIO_OSPEED_50MHZ, GPIO_PIN_10 | GPIO_PIN_11);

    i2c_clock_config(I2C1, 100000, I2C_DTCY_16_9);
    i2c_mode_addr_config(I2C1, I2C_I2CMODE_ENABLE, I2C_ADDFORMAT_7BITS, 1);
    i2c_enable(I2C1);
    i2c_ack_config(I2C1, I2C_ACK_ENABLE);
}

/************************* END ****************************/
