/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                 SFS_Port_SPI_MK20DX256.c                 *
*                 Teensy 3.2 SPI xRAM PORT                 *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_SPI_HWxxxx.h"


/**********************************************************
 *   Teensy 3.2 configuration SPI0
 ***********************************************************
 *   PTC5 (13) -------------------------------- SCK     ->
 *   PTC7 (12) -------------------------------- MISO    <-
 *   PTC6 (11) -------------------------------- MOSI    ->
 *   PTC4 (10) -------------------------------- /CS     ->  (manual used)
 **********************************************************/

/*
************************************************************
*             LOWLEVEL SET /CS TO ACTIVE
************************************************************
*/
void _sfsPort_CS_en(void)
{
#ifdef _pC_OS_
    OS_Lock();
#endif
//    GPIO_ResetBits(GPIOA,GPIO_Pin_4);
}

/*
************************************************************
*             LOWLEVEL RESET /CS TO DEACTIVE
************************************************************
*/
void _sfsPort_CS_dis(void)
{
//    GPIO_SetBits(GPIOA,GPIO_Pin_4);
#ifdef _pC_OS_
    OS_Unlock();
#endif
}

/*
************************************************************
*             LOWLEVEL BYTE_WISE I/O-FUNCTION
************************************************************
*/
U08 _sfsPort_RW(U08 byte)
{
    while(!(SPI0->SR & SPI_SR_TXCTR_MASK)) _NOP();
    SPI0->PUSHR = byte;
    while(!(SPI0->SR & SPI_SR_RXCTR_MASK)) _NOP();
    return (U08)(SPI0->POPR);
}

/*
************************************************************
*                LOWLEVEL READ STATUS
************************************************************
*/
U08 _sfsPort_readStatus(U08 cmd)
{
    U08 sr;

    _sfsPort_CS_en();
    _sfsPort_RW(cmd);
    sr = _sfsPort_RW(0xFF);
    _sfsPort_CS_dis();
    return sr;
}
/*
************************************************************
*                 INIT SPI-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    SIM->SCGC6    |= SIM_SCGC6_SPI0_MASK;
    PORTC->PCR[5]  = PORT_PCR_MUX(2);
    PORTC->PCR[4]  = PORT_PCR_MUX(2);
    PORTC->PCR[3]  = PORT_PCR_MUX(2);
    PORTC->PCR[2]  = PORT_PCR_MUX(1);   // CS is manully used
    PTC->PDOR      = (1UL <<  2);
    PTC->PDDR      = (1UL <<  2);
    PTC->PTOR      = (1UL <<  2);
    SPI0->CTAR[0]  = SPI_CTAR_FMSZ(0x7) | SPI_CTAR_CPOL_MASK;
    SPI0->CTAR[0] |= (1<<2) | 1;
    SPI0->CTAR[0] |= (1<<14);
    SPI0->MCR      = SPI_MCR_MSTR_MASK | SPI_MCR_PCSIS(0x1) | SPI_MCR_HALT_MASK;
/*
    SPIx->MCR     = (SPI_MCR_CLR_RXF_MASK | SPI_MCR_CLR_TXF_MASK | SPI_MCR_PCSIS_MASK | SPI_MCR_HALT_MASK);
    SPIx->MCR    |= SPI_MCR_MSTR_MASK;
    SPIx->CTAR[0] = (SPI_CTAR_DBR_MASK | SPI_CTAR_BR(7) | SPI_CTAR_FMSZ(7) | SPI_CTAR_CSSCK(4) | SPI_CTAR_ASC(4) | SPI_CTAR_DT(6) | SPI_CTAR_PDT_MASK | SPI_CTAR_CPOL_MASK | PI_CTAR_CPHA_MASK);
    SPIx->SR     |= (SPI_SR_EOQF_MASK | SPI_SR_TFFF_MASK | SPI_SR_TFUF_MASK | SPI_SR_RFDF_MASK | SPI_SR_RFOF_MASK);
    SPIx->MCR    &= ~SPI_MCR_HALT_MASK;
*/
}

/************************* END ****************************/
