/***********************************************************
*                       pC/SFS 1.14b                       *
*            a FREE - small Serial File System             *
*                                                          *
*           by Frank Goetze  -  www.embedded-os.de         *
************************************************************
*                   SFS_Port_I2C_SAMD51.c                  *
*                   SAMD51 I2C xRAM PORT                   *
************************************************************
* Revision-Info:                                           *
*                                                          *
***********************************************************/

#include "../../../inc/LLdriver_HW/SFS_Port_I2C_HWxxxx.h"

#ifdef __IAR_SYSTEMS_ICC__
 #include "Microchip/iosamd51.h"
#else
 #include <SAMD51.h>
#endif

#define SAMD51_TACHYON        0
#define SAMD51_ITSYBITSY_M4   1

#define SAMD51_TYPE           SAMD51_ITSYBITSY_M4

/***********************************************************
 *   Atmel SAMD51G18 configuration "TACHYON"
 ***********************************************************
 *   PA22 --- SERCOM3.0 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PA23 --- SERCOM3.1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are on board !!!
 **********************************************************/
#if (SAMD51_TYPE == SAMD51_TACHYON)
 #define I2C_SERCOMx   SERCOM3
#endif

/***********************************************************
 *   Atmel SAMD51G18 configuration "ItsyBitsy M4"
 ***********************************************************
 *   PA12 --- SERCOM2.0 ----------------------- SDA     ->    (NO internal pull-up possible !)
 *   PA13 --- SERCOM2.1 ----------------------- SCL     ->    (NO internal pull-up possible !)
 *
 *   !!! external pull-up's are required (2.2k) !!!
 **********************************************************/
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)
 #define I2C_SERCOMx   SERCOM2
#endif

#define I2C_MODE_MASTER   5

enum {
  I2C_TRANSFER_WRITE = 0,
  I2C_TRANSFER_READ  = 1,
};

/*
************************************************************
*                LOWLEVEL READ BYTES
************************************************************
*/
U08 _sfsPort_readBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    I2C_SERCOMx->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        I2C_SERCOMx->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    I2C_SERCOMx->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_READ;
    while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_SB)) _NOP();
    if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    I2C_SERCOMx->I2CM.CTRLB.reg &= ~SERCOM_I2CM_CTRLB_ACKACT;
    for(i = 0; i < (len-1); i++) {
        data[i] = I2C_SERCOMx->I2CM.DATA.reg;
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_SB)) _NOP();
    }
    I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_ACKACT;
    if(len) {
        I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        data[len-1] = I2C_SERCOMx->I2CM.DATA.reg;
    }
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WRITE BYTES
************************************************************
*/
U08 _sfsPort_writeBytes(U32 addr, U08 *data, SFS_LONG len)
{
    SFS_LONG i;

    I2C_SERCOMx->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        I2C_SERCOMx->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    for (i = 0; i < len; i++) {
        I2C_SERCOMx->I2CM.DATA.reg = data[i];
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL FILL BYTES
************************************************************
*/
U08 _sfsPort_fillBytes(U32 addr, U08 value, SFS_LONG len)
{
    SFS_LONG i;

    I2C_SERCOMx->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
    while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
    if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
        I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
        return SFS_PORT_ERR;
    }
    for (i = 2; i > 0; i--) {
        I2C_SERCOMx->I2CM.DATA.reg = (addr >> (8*(i-1))) & 0xFF;
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    for (i = 0; i < len; i++) {
        I2C_SERCOMx->I2CM.DATA.reg = value;
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        if (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK) {
            I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
            return SFS_PORT_ERR;
        }
    }
    I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    return SFS_NO_ERR;
}

/*
************************************************************
*                LOWLEVEL WAIT READY
************************************************************
*/
void _sfsPort_writeReady(void)
{
    U32 addr = 0;

    do {
        I2C_SERCOMx->I2CM.ADDR.reg = I2C_MEM_CC_ADDRESS | I2C_MEM_CS_ADDRESS | I2C_MEM_BLOCK | I2C_TRANSFER_WRITE;
        while (0 == (I2C_SERCOMx->I2CM.INTFLAG.reg & SERCOM_I2CM_INTFLAG_MB)) _NOP();
        I2C_SERCOMx->I2CM.CTRLB.reg |= SERCOM_I2CM_CTRLB_CMD(3);
    }
    while (I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_RXNACK);
}

/*
************************************************************
*                 INIT I2C-INTERFACE
************************************************************
*/
void _sfsPort_Init(void)
{
    I2C_SERCOMx->I2CM.CTRLA.bit.ENABLE = 0;
#if (SAMD51_TYPE == SAMD51_TACHYON)
    MCLK->APBBMASK.reg |= MCLK_APBBMASK_SERCOM3; 
    GCLK->PCHCTRL[SERCOM3_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN_GCLK1_Val | (1 << GCLK_PCHCTRL_CHEN_Pos); 
#endif
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)
    MCLK->APBBMASK.reg |= MCLK_APBBMASK_SERCOM2; 
    GCLK->PCHCTRL[SERCOM2_GCLK_ID_CORE].reg = GCLK_PCHCTRL_GEN_GCLK1_Val | (1 << GCLK_PCHCTRL_CHEN_Pos); 
#endif    
    I2C_SERCOMx->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SWRST;
    while(I2C_SERCOMx->I2CM.SYNCBUSY.bit.SWRST) __NOP();
    I2C_SERCOMx->I2CM.CTRLA.reg = SERCOM_I2CM_CTRLA_SPEED(1) | SERCOM_I2CM_CTRLA_SDAHOLD(0x2) | 
                              SERCOM_I2CM_CTRLA_RUNSTDBY | SERCOM_I2CM_CTRLA_MODE(I2C_MODE_MASTER);
    I2C_SERCOMx->I2CM.CTRLB.reg = SERCOM_I2CM_CTRLB_SMEN | SERCOM_I2CM_CTRLB_ACKACT;
    while(I2C_SERCOMx->I2CM.SYNCBUSY.bit.SYSOP) __NOP();
    I2C_SERCOMx->I2CM.BAUD.reg = SERCOM_I2CM_BAUD_BAUD(11) | SERCOM_I2CM_BAUD_BAUDLOW(22);
    while(I2C_SERCOMx->I2CM.SYNCBUSY.bit.SYSOP) __NOP();
#if (SAMD51_TYPE == SAMD51_TACHYON)
    PORT->Group[0].PMUX[PIN_PA22/2].bit.PMUXE  = 2;  // AF-C
    PORT->Group[0].PINCFG[PIN_PA22].bit.PMUXEN = 1;
    PORT->Group[0].PMUX[PIN_PA23/2].bit.PMUXO  = 2;  // AF-C
    PORT->Group[0].PINCFG[PIN_PA23].bit.PMUXEN = 1;
#endif
#if (SAMD51_TYPE == SAMD51_ITSYBITSY_M4)
    PORT->Group[0].PMUX[PIN_PA12/2].bit.PMUXE  = 2;  // AF-C
    PORT->Group[0].PINCFG[PIN_PA12].bit.PMUXEN = 1;
    PORT->Group[0].PMUX[PIN_PA13/2].bit.PMUXO  = 2;  // AF-C
    PORT->Group[0].PINCFG[PIN_PA13].bit.PMUXEN = 1;
#endif
    I2C_SERCOMx->I2CM.CTRLA.reg |= SERCOM_I2CM_CTRLA_ENABLE;
    while(I2C_SERCOMx->I2CM.SYNCBUSY.bit.ENABLE) __NOP();
    while(!(I2C_SERCOMx->I2CM.STATUS.reg & SERCOM_I2CM_STATUS_BUSSTATE(1))) {
        I2C_SERCOMx->I2CM.STATUS.reg = SERCOM_I2CM_STATUS_BUSSTATE(1); 
    }
}

/************************* END ****************************/
